from sirius_common_utils.core.logging import logging, log_name

logger = logging.get_logger(log_name.LOG_COMMON_CORE_PLATFORM)

import platform

def is_platform_arm():
    text = platform.machine();
    logger.debug(f"{text}")
    return text.startswith(("arm", "aarch64"))


def is_os_windows():
    return "windows" in platform.system()



from sirius_common_utils.core.logging import logging, log_name
logger = logging.get_logger(log_name.LOG_COMMON_CORE_ODBC_QUERY)


from sqlalchemy import create_engine, inspect

# Key: data source name
g_engine_map = {}


def get_engine(ds_name, server_name, db_name, username, password):
    global g_conn_map

    engine = g_engine_map.get(ds_name)
    if engine is None:
        engine = open_engine(server_name, db_name, username, password)
        g_engine_map[ds_name] = engine

    return engine


def open_engine(server_name, db_name, username, password):
    conn_str = f'mssql+pyodbc://{username}:{password}@{server_name}/{db_name}?driver=ODBC+Driver+17+for+SQL+Server'
    engine = create_engine(conn_str)
    return engine


def get_table_names(engine):
    inspector = inspect(engine)
    return inspector.get_table_names()


def close_engine(ds_name:None):
    global g_conn_map

    if ds_name is None:
        for conn in g_engine_map.values():
            conn.close()
    else:
        conn = g_engine_map[ds_name]
        if conn is not None:
            conn.close()



from datetime import datetime

PATTERN_DATETIME = "%d-%m-%Y %H:%M:%S"
PATTERN_DATETIME_TREND = "%Y-%m-%d %H:%M:%S"

PATTERN_DATE = "%d-%m-%Y"
PATTERN_TIME = "%H:%M:%S"

def convert_date_to_text(datetime:datetime, pattern=PATTERN_DATETIME, throws_ex_flag=False):
    return datetime.strftime(pattern)

def convert_text_to_date(text, pattern=PATTERN_DATETIME, throws_ex_flag=False):
    try:
        date_obj = datetime.strptime(text, pattern)
        return date_obj.date()
    except Exception as ex:
        if throws_ex_flag:
            raise ex
        else:
            return None


def convert_text_to_datetime(text, pattern=PATTERN_DATETIME, throws_ex_flag=False):
    try:
        date_obj = datetime.strptime(text, pattern)
        return date_obj
    except Exception as ex:
        if throws_ex_flag:
            raise ex
        else:
            return None



import time

from sirius_common_utils.core.logging import logging, log_name

logger = logging.get_logger(log_name.LOG_COMMON_CORE)
loggerWebInterceptor = logging.get_logger(log_name.LOG_COMMON_CORE_WEB_INTERCEPTOR)

from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse

from app.global_var import global_var

async def trace_log_filter_entrance(request: Request, call_next):
    start_time = time.time()
    loggerWebInterceptor.info(f"Request: {request.method} {request.url}")

    try:
        response = await call_next(request)
    except Exception as ex:
        # 全局异常处理
        loggerWebInterceptor.error(f"Error: {ex}", exc_info=True)
        response = JSONResponse(
            status_code=500,
            content={"errorCode": "500", "msg": str(ex)}
        )

    # 请求后逻辑
    process_time = time.time() - start_time
    response.headers["X-Process-Time"] = str(process_time)
    loggerWebInterceptor.info(f"Response status: {response.status_code} in {process_time:.4f} seconds")

    return response



from datetime import datetime


def success():
    return {
        "result": "success",
        "message": "success",
        "timestamp": datetime.now(),
    }


def failure(code='999', message=''):
    return {
        "result": "failure",
        "code": code,
        "message": message,
        "timestamp": datetime.now(),
    }



import time

from sirius_common_utils.core.logging import logging, log_name

logger = logging.get_logger(log_name.LOG_COMMON_CORE)
loggerWebInterceptor = logging.get_logger(log_name.LOG_COMMON_CORE_WEB_INTERCEPTOR)

from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse

from app.global_var import global_var




async def trace_log_filter_user_info(request: Request, call_next):
    # user = 'Joey'

    # try:
    response = await call_next(request)
    # finally:
    # loggerWebInterceptor.info(f"User Info: {user}")

    return response




from app.global_var import constant, log_name

import logging
import pyodbc


logger = logging.getLogger(log_name.LOG_COMMON_CORE_ODBC_CONNECTION)

# Key: conn name
g_conn_map = {}

def get_conn(ds_name, server_name, db_name, username, password):
    global g_conn_map

    conn = g_conn_map.get(ds_name)
    if conn is None:
        conn = open_conn(server_name, db_name, username, password)
        g_conn_map[ds_name] = conn

    return conn

def open_conn(server_name, db_name, username, password):
    conn_str = f'DRIVER={{ODBC Driver 17 for SQL Server}};SERVER={server_name};DATABASE={db_name};UID={username};PWD={password}'
    conn = pyodbc.connect(conn_str)
    return conn

def close_conn(ds_name:None):
    global g_conn_map

    if ds_name is None:
        for conn in g_conn_map.values():
            conn.close()
    else:
        conn = g_conn_map[ds_name]
        if conn is not None:
            conn.close()



from core.logging import logging, log_name
logger = logging.get_logger(log_name.LOG_COMMON_CORE_CLOUD_BIGQUERY)


from google.cloud import bigquery

from app.global_var import constant

import google.api_core.exceptions as core_exceptions

g_client = None
g_dataset = None
g_table_asset = None


def init():
    global g_client, g_dataset

    if g_client is None:
        g_client = bigquery.Client(project=constant.GOOGLE_PROJECT)

    if g_dataset is None:
        g_dataset = bigquery.Dataset(constant.GOOGLE_DATASET)
        create_dataset()


def client():
    return g_client


def create_dataset():
    global g_client, g_dataset

    g_dataset.location = constant.GOOGLE_REGION
    g_dataset.description = "sirius"
    g_dataset.labels = {"environment": "production", "type": "timeseries"}

    g_dataset = g_client.create_dataset(bigquery.Dataset(constant.GOOGLE_DATASET), exists_ok=True)
    logger.info(f"Dataset {g_dataset.full_dataset_id} created in project {g_dataset.project}.")


def delete_dataset():
    global g_client, g_dataset

    g_client.delete_dataset(g_dataset, True)
    logger.info(f"Dataset '{g_dataset.full_dataset_id}' deleted successfully.")


def dataset():
    return g_dataset


def create_table(table_id, schema, re_create_flag):
    global g_client, g_dataset

    if not isinstance(table_id, str):
        raise ValueError("Input must be a tableId string")

    is_table_exist = exist_table(table_id)

    if is_table_exist:
        if re_create_flag:
            delete_table(table_id)
        else:
            return bigquery.Table(table_id)

    table = g_client.create_table(bigquery.Table(table_id, schema=schema))
    logger.info(f"Table '{table.full_table_id}' created successfully.")

    return table


def exist_table(tableId):
    global g_client, g_dataset

    try:
        g_client.get_table(tableId)
        return True
    except core_exceptions.NotFound:
        return False


def delete_table(table_id):
    global g_client

    # g_client.delete_table(bigquery.Table(tableId), True)
    g_client.Table(table_id).delete()
    # logger.info(f"Table '{table.full_table_id}' deleted successfully.")


def list_tables():
    global g_client, g_dataset
    tables = g_client.list_tables(g_dataset)

    for table in tables:
        print(table.table_id)


def delete_table(tableId):
    global g_client, g_table_trenddata

    g_client.delete_table(bigquery.Table(tableId))
    logger.info(f"Table '{tableId}' deleted successfully.")





from . import interceptor_entrance
from . import interceptor_user_info

def register_middlewares(app):
    app.middleware("http")(interceptor_entrance.trace_log_filter_entrance)
    app.middleware("http")(interceptor_user_info.trace_log_filter_user_info)



import json
import time

from core.datetime import date_formatter

import paho.mqtt.client as mqtt

from app.global_var import constant

from sirius_common_utils.core.logging import logging, log_name

logger = logging.get_logger(log_name.LOG_COMMON_CORE_MQTT)

g_mqtt_broker = None
g_mqtt_port = None


g_mqtt_client = None
g_status = {
    "ConnStatus": "",
    "Date": 0,
}


def init(broker, port):
    global g_mqtt_broker, g_mqtt_port

    g_mqtt_broker = broker
    g_mqtt_port = port

    get_client()


def connect():
    client = mqtt.Client()
    client.on_connect = on_connect
    client.on_message = on_message

    try:
        logger.debug(f"Establish MQTT Connection to {g_mqtt_broker}:{g_mqtt_port}")
        client.connect(g_mqtt_broker, g_mqtt_port, 60)

        client.loop_start()
    except Exception as e:
        print(f"连接失败：{e}")

    return client


def get_client():
    global g_mqtt_client

    if g_mqtt_client is None:
        g_mqtt_client = connect()

    return g_mqtt_client


def on_connect(client, userdata, flags, rc):
    global g_status

    if rc == 0:
        g_status['ConnStatus'] = 'Connected'
        logger.info("Establish MQTT Connection success")
    else:
        g_status['ConnStatus'] = 'Disconnected'
        logger.error(f"Establish MQTT Connection failed: {rc}")

    g_status['Date'] = date_formatter.convert_date_to_text(time, "%Y%m%d%H%M%S")

    msg = json.dumps(g_status)
    logger.debug(f"Send MQTT Status: {msg}")

    client.publish(constant.MQTT_TOPIC_STATUS, msg)


def on_message(client, userdata, msg):
    logger.error(f"Receive MQTT message from Topic: {msg.topic} without any handler, msg: {msg}")



import json
import traceback
from typing import Callable, Dict

from app.mqtt.connection import get_client

from sirius_common_utils.core.logging import logging, log_name

logger = logging.get_logger(log_name.LOG_COMMON_CORE_MQTT)


def init():
    return

def subscribe(topic_name: str, qos: int, handler: Callable[[Dict], None]):
    client = get_client()

    def message_callback(client, user_data, msg):
        logger.debug(f"Receive message from MQTT Topic {topic_name}: {msg.payload}")

        data = None
        try:
            payload = msg.payload.decode()
            data = json.loads(payload)
        except Exception as e:
            logger.error(f"Cannot parse the JSON message from MQTT Topic {topic_name}, msg: {msg.payload}, exception: {e}")

        try:
            handler(data)
        except Exception as e:
            logger.error(f"Handle MQTT Topic {topic_name}, msg: {msg.payload} with error: {e}")
            traceback.print_exc()

    client.message_callback_add(topic_name, message_callback)
    client.subscribe(topic_name, qos)
    logger.info(f"subscribe MQTT topic: {topic_name}")




from app.global_var import constant, log_name

import logging

def create(ds_name:None):
    global g_engine_map

    if ds_name is None:
        for conn in g_engine_map.values():
            conn.close()
    else:
        conn = g_engine_map[ds_name]
        if conn is not None:
            conn.close()



from datetime import datetime

def generate_version():
    now = datetime.now()
    version = now.strftime("%Y%m%d%H%M%S")
    return "0.0."+version

import shutil
import os

if os.path.exists("dist"):
    shutil.rmtree("dist")


from setuptools import setup, find_packages

setup(
    name="sirius-common-utils",  # 包名
    version=generate_version(),  # 版本号
    author="Joey Yang",  # 作者名
    author_email="156838991@qq.com",  # 作者邮箱
    description="sirius-common-utils",  # 简短描述
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/my_package",  # 项目主页
    package_dir={"": "src"},  # 指定源代码目录
    packages=find_packages(where="src"),  # 自动发现包
    classifiers=[  # 分类信息
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',  # 依赖的 Python 版本
    install_requires=[  # 依赖项
        "requests",
        "numpy"
    ],
)



from sirius_common_utils.core.logging import logging, log_name
logger = logging.get_logger(log_name.LOG_COMMON_CORE)

from pathlib import Path

import logging

g_project_root = None

def init():
    global g_project_root

    g_project_root = Path(__file__).resolve().parent.parent.parent.parent
    logger.debug(f"project_root: {g_project_root}")
    logger.debug(f"init")


def get_project_root():
    global g_project_root

    if g_project_root is None:
        g_project_root = Path(__file__).resolve().parent.parent.parent.parent
        logger.debug(f"project_root: {g_project_root}")

    return g_project_root


LOG_COMMON_CORE = 'common.core'
LOG_COMMON_CORE_WEB_INTERCEPTOR = 'common.core.web.interceptor'

LOG_COMMON_CORE_ODBC_CONNECTION = 'common.core.odbc.connection'
LOG_COMMON_CORE_ODBC_QUERY = 'app.core.odbc.query'

LOG_COMMON_CORE_CLOUD_BIGQUERY = 'app.core.cloud.bigquery'

LOG_COMMON_CORE_MQTT = 'common.core.mqtt'
LOG_COMMON_CORE_FETCHER = 'common.core.fetcher'
LOG_COMMON_CORE_GPIO = 'common.core.gpio'
LOG_COMMON_CORE_PLATFORM = 'common.core.platform'

LOG_APP_TREND = 'app.trend'
LOG_APP_ASSET = 'app.asset'
